/*********************************************************************                                                                   **
**  Device:   A8137-F4
**  File:			main.c
**  Target:	
**  Tools:		ICE 
**  Created:	2020-06-12	 
**  Description: A8106 reference code for FIFO mode
**  This file is a sample code for your reference.
**	  
**  Copyright (C) 2017 AMICCOM Corp.
**	 
**  Revision history:
**  
*********************************************************************/
#define _MAIN_

#include "main.h"	
#include "globalvar.h"
#include "A8137F4LIB_RF_22_10_2021.h"
#include "A8137F4.h"
#include "Uti.h"
#include "SPI.h"

//#define _ADC_
//#define _RSSI_
#define _PM_MODE_
//#define _TWOR_
//#define _WOT_
#define _WOR_
//#define _CEFCC_TX_
//#define _CEFCC_RX_
//#define _CEFCC_DIRECT_
//#define _ENABLE_RTC_
//#define _SLEEP_TIMER_

/*********************************************************************
**						select data rate  							**
*********************************************************************/
#define DR_2Mbps_dev_500k
//#define DR_1Mbps_dev_250k

#define GIO1	P0_7
#define GIO2	P1_2


/*********************************************************************
**  											function Declaration											**
*********************************************************************/
void InitTimer0(void);
void Timer0ISR (void);
void Uart0Isr(void);
void InitUart0Timer2(void);
void RxPacket(void);
void GIOPin(Uint8);
void WriteFIFO(void);
void ErrState(void);
void UserRegister(void);

Uint16 ADC_measurement_enable(Uint8 Input_Pin, Uint8 ADC_On);
Uint8 RSSI_measurement_enable(void);
void A8137_PM(Uint8 mode);
void TWOR(Uint32 rbt);
void WOT(Uint32 rbt, Uint8 wtlp);
void WOR(Uint32 rbt, Uint16 rbto, Uint8 wus);
void CEFCC_TX(Uint8 Channel, Uint8 TXCS, Uint8 PAC, Uint8 TBG);
void FCC_CE_RX(Uint8 Channel, Uint8 TXCS, Uint8 PAC, Uint8 TBG);
void FCC_CE_Direct(Uint8 Channel, Uint8 TXCS, Uint8 PAC, Uint8 TBG);
void Enable_RTC(void);
void SLEEP_Timer(Uint8 RTB0, Uint8 RTB1, Uint8 RTB2);

/*********************************************************************
** 														main loop  		                        **
*********************************************************************/
void main(void)
{	
	/*	initHW	*/
	Uint8 i;
	P0 = 0xFF;
	P1 = 0xFF;
	P3 = 0xFF;

	P0OE = 0xFF;
	P1OE = 0xFF;
	P3OE = 0xFF;

	P0PUN = 0xFF;
	P1PUN = 0xFF;
	P3PUN = 0xFF;
	
	GIOPin(ON);
	
	InitTimer0();
	InitUart0Timer2();
	
	CalResault = RFLIB_InitRF();
	if (CalResault)
		ErrState();
	
	UserRegister();
	RFLIB_StrobeCmd(CMD_STBY);
	WriteID();

	EIE |= 0x08;   //Enable RF Interrupt
	TR0 = 1;	//	enable timer0
	EA = 1;		//	enable global interrupt
	
	RFLIB_SetCH(100);	//freq 2450MHz
	
	if (P3_5) 
	{
		
		SSCR = 0xFF;    // SCS=1
		SPSR = 0x00;    // SSEN=0;
		ESPI = ENABLE;  // enable SPI interrupt
		EA = ENABLE;
		
		ESPI = DISABLE;  // disable SPI interrupt
		SPCR = SPI_SPE;//SPI interrupt disable, SPI enable, slave mode
		SPICnt=0;
		for(i = 0; i < 64; i++)
			SPIBuf[i]=0;
		
		while (1) 
		{
			while(SPICnt!=64)
			{
				while ( P0_3 == LOW ) 
					_nop_(); // wait SSO turn high
				SPDR =0x00;
				while ( ( SPSR & SPI_SPIF ) == 0 );
				SPIBuf[SPICnt++] = SPDR;
			}
			SPICnt=0;
			
			//TEST LED
			for(i = 0; i < 64; i++)
			{
				if(SPIBuf[i]==i)
				{
					P1_7=0;
				}
				else
				{
					P1_7=1;
					break;
				}
			}
			
			
			//write FIFO 
			for(i = 0; i < 64; i++)
				RFLIB_WriteReg(TXFIFO_REG + i , SPIBuf[i]);
			
			
//			WriteFIFO();
			for(i = 0; i < 100; i++)
			{
				RF_Flag = 0;
				RFLIB_StrobeCmd(CMD_TX); //entry tx & transmit
				Delay10us(1);
				while(RF_Flag == 0); //wait transmit completed
//				Delay10ms(1);
			}
			P1_1=0;
		}		
	} 
	
	else 
	{
		RxCnt = 0;
		Err_BitCnt = 0;
		LossCnt = 0;
		
		

		
		while (1) 
		{
			


//			WOR(Uint32 rbt, Uint16 rbto, Uint8 wus);
//******************************************************************************
//			WOR(Uint32 rbt, Uint16 rbto, Uint8 wus);
//			Rx time = (rbto+1)*320us
//			sleep time = (rbt+1)*320us - RX time
//          Wus=1  : Detect IEEE 802.15.4 Sync word(Detect ID wake up MCU)
//******************************************************************************			
			WOR(24, 6, 1);
			
			P1OE=0xFF;
			
			RF_Flag = 0;
			RFLIB_StrobeCmd(CMD_RX);
			P0_6=1;
			Delay10us(1);		
			while(RF_Flag == 0); //wait receive completed
			RxPacket();
			
			P1_1=~P1_1;

			//TEST LED
//			for(i = 0; i < 64; i++)
//			{
//				if(Tmpbuf[i]==i)
//				{
//					P1_7=0;
//				}
//				else
//				{
//					P1_7=1;
//					while(1);
//					break;
//				}
//			}
			
			SSCR = 0xFF;    // SCS=1
			SPSR = 0x00;    // SSEN=0;
			ESPI = ENABLE;  // enable SPI interrupt
			EA = ENABLE;
			
			ESPI = DISABLE;  // disable SPI interrupt
			SPCR = SPI_SPE;//SPI interrupt disable, SPI enable, slave mode
			SPICnt=0;
			
			P0_6=0;
//			P1_1=1;
			SPDR =Tmpbuf[SPICnt];
			while(SPICnt!=64)
			{
				while ( P0_3 == LOW ) 
				_nop_(); // wait SSO turn high
				SPDR =Tmpbuf[SPICnt++];
				while ( ( SPSR & SPI_SPIF ) == 0 );
//				Tmpbuf[SPICnt++] = SPDR;
			}

		
			SPCR &= ~SPI_SPE;//disable SPI
			SPICnt=0;
			Delay1ms(250);
			
		}
	}
}

/************************************************************************
**  							InitTimer0							 **	
************************************************************************/
void InitTimer0(void)
{
	TR0 = 0;
	TMOD =(TMOD & 0xF0)|0x01;	// timer0 mode=1
	TH0 = (65536 - t0hrel)>>8;	// Reload Timer0 high byte,low byte
	TL0 = 65536 - t0hrel;
	TF0 = 0;					// Clear any pending Timer0 interrupts
	ET0 = 1;					// Enable Timer0 interrupt
}

/************************************************************************
**  						InitUart0Timer2                           **
************************************************************************/
void InitUart0Timer2(void)
{
	IOSEL |= 0x01;//enable BBIOS + UART
	PCON = 0x00;
	SCON = 0x40;
	T2CON = 0x30;
	RCAP2H = BaudRate38400 / 256;
	RCAP2L = BaudRate38400 % 256;
	TR2=1;
	REN = 1;
	ES=1;//enable uart0 interrupt
}


/************************************************************************			    
** 								GIOPin	                             **    
************************************************************************/
void GIOPin(Uint8 src)
{
    if(src)
		IOSEL |= 0x08;
    else
		IOSEL &= 0xF7;
}

/************************************************************************			    
** 								  	    User set register                            **    
************************************************************************/
void UserRegister(void) {
	Uint8 n;
	
	RFLIB_WriteReg(STROBECTRL_REG, 0x00);
	RFLIB_WriteReg(MODECTRL_REG, 0x02);
	RFLIB_WriteReg(MODESEL_REG, 0x00);
	RFLIB_WriteReg(CALIBRATION_REG, 0x00);
	
	RFLIB_WriteReg(PHR_REG, 0x3F);
	
	RFLIB_WriteReg(TWUN_REG, 0x00);
	
	RFLIB_WriteReg(RCOSC1_REG, 0x82);

	RFLIB_WriteReg(CKO_REG, 0xF8);
	RFLIB_WriteReg(GIO1_REG, 0x04);
	RFLIB_WriteReg(GIO2_REG, 0x04);
	
	RFLIB_WriteReg(PLL1_REG, 0x50);
	
	RFLIB_WriteReg(TX1_REG, 0x16);
	
	RFLIB_WriteReg(ADC_REG, 0x00);
	RFLIB_WriteReg(ADCC_REG, 0x63);
	
	RFLIB_WriteReg(BATTERY_REG, 0x00);
	
	RFLIB_WriteReg(TXTEST_REG, 0x1F);
	RFLIB_WriteReg(CRYSTALTEST_REG, 0xED);
	
	RFLIB_WriteReg(CSMA_REG, 0x64);
	RFLIB_WriteReg(BE_REG, 0x53);
	
	RFLIB_WriteReg(INTST_REG, 0x10);
	
	RFLIB_WriteReg(ADFC1_REG, 0x7C);
	RFLIB_WriteReg(ADFC2_REG, 0xF4);
	
	for (n = 0; n < 12; n++) 
		RFLIB_WriteReg(ADFF_REG + n, 0x00);
	
	RFLIB_WriteReg(ACKFIFO_REG + 0, 0x02);
	RFLIB_WriteReg(ACKFIFO_REG + 1, 0x00);
	
	RFLIB_WriteReg(PNG_REG + 0, 0x81);
	RFLIB_WriteReg(PNG_REG + 1, 0x25);
	
	RFLIB_WriteReg(RBTO_REG + 0, 0x00);
	RFLIB_WriteReg(RBTO_REG + 1, 0x00);
	
	for (n = 0; n < 3; n++)  
		RFLIB_WriteReg(RBT_REG + n, 0x00);
	
	RFLIB_WriteReg(CODE_REG, 0x00);
	
	RFLIB_WriteReg(CRCRIV_REG + 0, 0xFF);
	RFLIB_WriteReg(CRCRIV_REG + 1, 0xFF);
	
	RFLIB_WriteReg(CRCTIV_REG + 0, 0x1D);
	RFLIB_WriteReg(CRCTIV_REG + 1, 0x0F);
	
	RFLIB_WriteReg(CRCPL_REG + 0, 0x10);
	RFLIB_WriteReg(CRCPL_REG + 1, 0x21);
	
	RFLIB_WriteReg(WS_REG, 0x2A);
	
	RFLIB_WriteReg(BDC_REG, 0x04);
	
	RFLIB_WriteReg(SDR_REG, 0x00);
		
	#ifdef DR_2Mbps_dev_500k
		RFLIB_WriteReg(0x0810, 0x9D);
		RFLIB_WriteReg(0x081B, 0x40);
		RFLIB_WriteReg(0x0828, 0x10);
		RFLIB_WriteReg(0x082A, 0x83);
		RFLIB_WriteReg(0x082B, 0x37);
		RFLIB_WriteReg(0x082D, 0x2B);
		RFLIB_WriteReg(0x0854, 0xB8);
	#endif
	
	#ifdef DR_1Mbps_dev_250k
		RFLIB_WriteReg(0x0810, 0x5D);
		RFLIB_WriteReg(0x081B, 0x20);
		RFLIB_WriteReg(0x0828, 0x00);
		RFLIB_WriteReg(0x082A, 0x80);
		RFLIB_WriteReg(0x082B, 0x17);
		RFLIB_WriteReg(0x082D, 0x0B);
		RFLIB_WriteReg(0x0854, 0xB0);
	#endif
}

/*********************************************************************
** 							WriteFIFO								*
*********************************************************************/
void WriteFIFO(void)
{
    Uint8 i;

    for(i = 0; i < 64; i++)
        RFLIB_WriteReg(TXFIFO_REG + i , PN9_Tab[i]);
}

/*********************************************************************
** 							WriteID							*
*********************************************************************/
void WriteID(void)
{
	Uint8 i;

	//Write ID
	for (i = 0; i < sizeof(ID_Tab); i++)
			RFLIB_WriteReg(SID_REG + i, ID_Tab[i]);
}

/*********************************************************************
** 																		RxPacket																				*
*********************************************************************/
void RxPacket(void)
{
	Uint8 i,recv,tmp;
	Uint8 err;

	err = 0;
  RxCnt++;
  for(i=0; i < 64; i++)
  {
		recv = RFLIB_ReadReg(RXFIFO_REG + i);
		Tmpbuf[i]=recv;
		if((recv ^ PN9_Tab[i])!=0)
		{
	    Err_ByteCnt++;
	    tmp = recv ^ PN9_Tab[i];
	    Err_BitCnt += (BitCount_Tab[tmp>>4] + BitCount_Tab[tmp & 0x0F]);

	    err=1;
		}
   }

  if (err)//packet error
	LossCnt++;	
}

/************************************************************************
**  Timer0ISR
************************************************************************/
void Timer0ISR (void) interrupt 1
{
	TF0 = 0;					// Clear Timer0 interrupt
	TH0 = (65536-t0hrel)>>8;	// Reload Timer0 high byte,low byte
	TL0 = 65536-t0hrel;

	Timer++;
	if (Timer == TIMEOUT)
			TimeoutFlag=1;

	TimerCnt0++;
	if (TimerCnt0 == 500)
	{
		TimerCnt0=0;
		CmdBuf[0]=0xF1;

		memcpy(&CmdBuf[1], &RxCnt, 2);
		memcpy(&CmdBuf[3], &SubRxCnt, 1);
		memcpy(&CmdBuf[4], &LossCnt, 1);
		memcpy(&CmdBuf[7], &Err_BitCnt, 4);
		memcpy(&CmdBuf[11], &Err_Frame, 1);

		UartSendCnt=12;
		Uartptr=&CmdBuf[0];
		SBUF=CmdBuf[0];
	}
}

/************************************************************************
**  Uart0 ISR
************************************************************************/
void Uart0Isr(void) interrupt 4 using 3
{
	if (TI==1)
	{
		TI=0;
		UartSendCnt--;
		if(UartSendCnt !=0)
		{
			Uartptr++;
			SBUF = *Uartptr;
		}
	}
	
	if (RI==1)
	{
		RI=0;
	}  
}

/************************************************************************
**  RF ISR
************************************************************************/
void RF_ISR(void) interrupt 10
{
    Uint8 tmp;
    Uint8 INT, IST, INTF;

    EIF = 0x08;  //clear flag

    tmp = RFLIB_ReadReg(INTST_REG);
  
    INTF = (tmp & 0x01) >> 0; //interrupt flag
    IST = (tmp & 0x78) >> 3;  //interrupt source
    INT = (tmp & 0x80) >> 7;  //interrupt state

    RFLIB_WriteReg(INTST_REG, 0x80);  //interrupt state reset

    //     IST     Interrupt source     INTF     Note
    //------------------------------------------------------------------------------------------------------------------
    //     000     none                 none   
    //     001     WTR                  CRCF
    //     002     CSMA_CA              CSMAF
    //     003     CCA                  CCAF
    //     004     ART                  ARTF
    //     005     EDM                  none
    //     006     FPFINT               FPF      FPFIS=1
    //     007     ADCM                 none
    //     008     WOR                  CRCF
    //     009     TWOR                 none
    //     010     WOT                  none     The interrupt happens before last TX, but IST is updated when WOT end
    
	if (IST == 1)       	
		RF_Flag = 1; 	                
#ifdef _SLEEP_TIMER_
	if (IST == 9) 
	{          
		SleepTimer_Flag = 1;
	}
#endif
}

/*********************************************************************
** 							Err_State																				*
*********************************************************************/
void ErrState(void)
{
	//ERR display
	//Error Proc...
	//...
	while(1);
}

/************************************************************************
** 							ADC measuremnt							 **
************************************************************************/
#ifdef _ADC_
Uint16 ADC_measurement_enable(Uint8 Input_Pin, Uint8 ADC_On)
{
	Uint8   ADC_8BitsValue;    
	
	RFLIB_WriteReg(ADCC_REG, 0x13);	// 16-average mode

	switch (Input_Pin) {
		case 0:	
			P3OE &= ~0x04;
			P3PUN &= ~0x04;
			break;
		
		case 1:
			P3OE &= ~0x08;
			P3PUN &= ~0x08;
			break;
		
		case 2:
			P3OE &= ~0x10;
			P3PUN &= ~0x10;
			break;
		
		case 3:
			P3OE &= ~0x20;
			P3PUN &= ~0x20;
			break;		
		
		default:
			P3OE &= ~0x04;
			P3PUN &= ~0x04;
	}

	IOSEL |= (Input_Pin << 6) | (ADC_On << 5);   

	RFLIB_WriteReg(MODECTRL_REG, 0x02 | 0x01);	// enable ADCM
	while (RFLIB_ReadReg(MODECTRL_REG) & 0x01);	// wait ADC finish

	ADC_8BitsValue = RFLIB_ReadReg(ADC_REG);

	return ADC_8BitsValue;		
}
#endif

/************************************************************************			    
** 						RSSI_measurement_enable                      **    
************************************************************************/
#ifdef _RSSI_
Uint8 RSSI_measurement_enable(void)
{
	Uint8 RSSI_Value;

	RFLIB_WriteReg(ADCC_REG, 0x73); //ADCOM -> ED hold after sync 128us

	RF_Flag = 0;
	RFLIB_StrobeCmd(CMD_RX); //entry rx
	Delay10us(1);
	while(RF_Flag == 0 && TimeoutFlag == 0); //wait receive completed

	if(TimeoutFlag)
		RFLIB_WriteReg(ADCC_REG, 0x63); //ADCOM -> ED no hold

	RSSI_Value = RFLIB_ReadReg(ADC_REG);  //Read RSSI (Signal RSSI when received data; environment RSSI when Timeout)
	RFLIB_StrobeCmd(CMD_STBY); //exit rx mode

	return RSSI_Value; 
}
#endif

/************************************************************************
**                               PM_MODE                               **
************************************************************************/
#ifdef _PM_MODE_
void A8137_PM(Uint8 mode)
{
	P0=0x00;
	P1=0x00;
	P3=0x00;
	P0OE=0xFF;
	P1OE=0xFF;
	P3OE=0xFF;

	
	RFLIB_StrobeCmd(CMD_SLEEP);
	switch (mode)
	{
		case 1:	// PM1 
			PCONE = 0x40; //PM1	
		  break;
		
		case 2:// PM2
			PCONE = 0x20; //PM2
		  break;

		case 3:	// PM3 
			PCONE = 0x08; //PM3
			break;

		default:
			PCONE = 0x40; //PM1	
			break;
	}	
	PCON=0x02;
	_nop_();
	_nop_();
	_nop_();
	_nop_();
	_nop_();
	_nop_();
	
	return;
}
#endif

#ifdef _TWOR_
/************************************************************************
**												TWWS(TWOR) function													 **
************************************************************************/
void TWOR(Uint32 rbt) {
	Uint8 rbt1, rbt2, rbt3;

	RFLIB_WriteReg(RCOSC1_REG, 0x82 | 0x08);  //Select external RTC

	rbt1 = (rbt & 0x000000FF) >> 0;
	rbt2 = (rbt & 0x0000FF00) >> 8;
	rbt3 = (rbt & 0x00FF0000) >> 16;

	RFLIB_WriteReg(RBT_REG + 0, rbt1);  //RBT -> Wakeup time
	RFLIB_WriteReg(RBT_REG + 1, rbt2);  //RBT -> Wakeup time
	RFLIB_WriteReg(RBT_REG + 2 ,rbt3);  //RBT -> Wakeup time  

	RFLIB_WriteReg(RSTCTRL_REG, BFC_RST);  //Back-off counter reset
	RFLIB_WriteReg(RCOSC1_REG, 0x82 | 0x01);  //Enable TWOR
	
	return;
}
#endif

#ifdef _WOT_
/************************************************************************
**															WOT function													 **
************************************************************************/
void WOT(Uint32 rbt, Uint8 wtlp) {
	Uint8 rbt1, rbt2, rbt3;

	rbt1 = (rbt & 0x000000FF) >> 0;
	rbt2 = (rbt & 0x0000FF00) >> 8;
	rbt3 = (rbt & 0x00FF0000) >> 16;

	RFLIB_WriteReg(RBT_REG + 0, rbt1);  //RBT -> Wakeup time
	RFLIB_WriteReg(RBT_REG + 1, rbt2);  //RBT -> Wakeup time
	RFLIB_WriteReg(RBT_REG + 2 ,rbt3);  //RBT -> Wakeup time

	if(wtlp > 15)
		wtlp = 15;

	RFLIB_WriteReg(BDC_REG, 0x04 | (wtlp << 4)); //WRLP -> loop times

	RFLIB_WriteReg(RSTCTRL_REG, BFC_RST);  //Back-off counter reset
	RFLIB_WriteReg(MODECTRL_REG, 0x02 | 0x20);  //Enable WOT
	
	return;
}
#endif

#ifdef _WOR_
/************************************************************************
**															WOR function													 **
************************************************************************/
//    WWS_AL      WWS_AL
//     ____ WWS_CL ____ WWS_CL ____
//____|    |______|    |______|    |_____
void WOR(Uint32 rbt, Uint16 rbto, Uint8 wus) {
	Uint8 rbt1, rbt2, rbt3;
	Uint8 rbto1, rbto2;

	RFLIB_WriteReg(GIO1_REG, 0x08); //GIO1=WOR
	RFLIB_WriteReg(GIO2_REG, 0x40); //GIO2=PDN_RX
	
	rbt1 = (rbt & 0x000000FF) >> 0;
	rbt2 = (rbt & 0x0000FF00) >> 8;
	rbt3 = (rbt & 0x00FF0000) >> 16;

	rbto1 = (rbto & 0x00FF) >> 0;
	rbto2 = (rbto & 0xFF00) >> 8;

	RFLIB_WriteReg(RBT_REG + 0, rbt1);  //RBT -> Wakeup time
	RFLIB_WriteReg(RBT_REG + 1, rbt2);  //RBT -> Wakeup time
	RFLIB_WriteReg(RBT_REG + 2 ,rbt3);  //RBT -> Wakeup time

	RFLIB_WriteReg(RBTO_REG + 0, rbto1);  //RBTO -> RX Active time
	RFLIB_WriteReg(RBTO_REG + 1, rbto2);  //RBTO -> RX Active time

	if(wus > 3)
			wus = 3;

	RFLIB_WriteReg(RCOSC1_REG, 0x82 | (wus << 4));  //WUS -> wakeup mode

	RFLIB_WriteReg(RSTCTRL_REG, BFC_RST);  //Back-off counter reset

	if(wus == 3)
		  RFLIB_WriteReg(MODECTRL_REG, 0x02 | 0x08 | 0x80);  //Enable WOR, Enable DFCRC
	else
	    RFLIB_WriteReg(MODECTRL_REG, 0x02 | 0x08);  //Enable WOR
	
	A8137_PM(2);
	while(GIO2==0);
	RFLIB_WriteReg(MODECTRL_REG, 0x02);  //disable WOR
	
	return;
}
#endif

#ifdef _CEFCC_TX_
/************************************************************************
**							 CEFCC_TX								**
************************************************************************/
void CEFCC_TX(Uint8 Channel, Uint8 TXCS, Uint8 PAC, Uint8 TBG) {
	RFLIB_SetCH(Channel);		
	RFLIB_WriteReg(MODECTRL_REG, 0x02);				//	FIFO mode
	RFLIB_WriteReg(TXTEST_REG, TXCS | PAC | TBG);	//	TX power TXSM[1:0]:Bit7-Bit6, 
	RFLIB_StrobeCmd(CMD_PLL);
	
	while (1) {
		RF_Flag = 0;
		RFLIB_StrobeCmd(CMD_TX);
		Delay10us(1);
		while(RF_Flag==0); //wait transmit completed	
	}
}
#endif

#ifdef _CEFCC_RX_
/************************************************************************
**							CEFCC_RX								 **
************************************************************************/
void FCC_CE_RX(Uint8 Channel, Uint8 TXCS, Uint8 PAC, Uint8 TBG) {
	RFLIB_SetCH(Channel);		
	RFLIB_WriteReg(MODECTRL_REG, 0x02);							//	FIFO mode
	RFLIB_WriteReg(TXTEST_REG, TXCS | PAC | TBG);	//	TX power TXSM[1:0]:Bit7-Bit6, 
	RFLIB_StrobeCmd(CMD_PLL);
	
	while (1) {
		RFLIB_StrobeCmd(CMD_RX);
	}
}
#endif

#ifdef _CEFCC_DIRECT_
/************************************************************************
** 							_CEFCC_DIRECT_							**
************************************************************************/
void FCC_CE_Direct(Uint8 Channel, Uint8 TXCS, Uint8 PAC, Uint8 TBG) {
	RFLIB_SetCH(Channel);		
	RFLIB_WriteReg(MODECTRL_REG, 0x00);				//	Direct mode
	RFLIB_WriteReg(TX1_REG, 0x06);					//	disable TME
	RFLIB_WriteReg(TXG_REG, 0x38);					//	disable TMDE
	RFLIB_WriteReg(TXTEST_REG, TXCS | PAC | TBG);	//	TX power TXSM[1:0]:Bit7-Bit6, 
	RFLIB_StrobeCmd(CMD_PLL);
	
	while (1) {
		RFLIB_StrobeCmd(CMD_TX);
	}
}
#endif

#ifdef _ENABLE_RTC_
/************************************************************************
**                            enable RTC                               **
**                                                                     **
** Turn on external RTC crystal.                                       **
**                                                                     **
************************************************************************/
void Enable_RTC(void) {
	IOSEL |= 0x10;	// Enable RTC pad
	P3OE &= ~0x40;
	P3PUN &= ~0x40;
	
	RFLIB_WriteReg(RCOSC1_REG, 0x82 | 0x08);  //Select external RTC
}	
#endif

#ifdef _SLEEP_TIMER_
/************************************************************************
**                             SLEEP Timer                             **
**                                                                     **
** User need run Enable_RTC() before run SLEEP_Timer()                 **
** to turn on the clock source of SLEEP_Timer()                        **
************************************************************************/
void SLEEP_Timer(Uint8 RTB0, Uint8 RTB1, Uint8 RTB2)
{	
	Enable_RTC();//Enable RTC
	
	RFLIB_WriteReg(RCOSC1_REG, 0x8A);// enable TMRE
	Delay10ms(20);

	//Sleep Timer counter RTB=RTB2[23:16]+RTB1[15:8]+RTB0[7:0]+1
	//Sleep Time: RTB*10/RTC
	RFLIB_WriteReg(RBT_REG + 0, RTB0);
	RFLIB_WriteReg(RBT_REG + 1, RTB1);
	RFLIB_WriteReg(RBT_REG + 2, RTB2);

	RFLIB_WriteReg(INTST_REG, 0x80);//interrupt state reset
	RFLIB_WriteReg(RSTCTRL_REG, 0x04);// Reset Back-off counter to zero.
	RFLIB_WriteReg(RCOSC1_REG, 0x8B);// enable TMRE
	EIF = 0x08;  //clear flag
	EIE |= 0x08;   //Enable RF Interrupt
	EA=1;
 		 	
	A8137_PM(2);	

	if (SleepTimer_Flag)
	{
		SleepTimer_Flag=0;
	}
	
	return;
}
#endif

